function M = getGPT(D, lambda, m, n)
% M = getGPT(D, lambda, m, n) returns the 2x2 matrix M_{mn} with elements
% M_{mn} = ( M_{mn}^{cc}   M_{mn}^{cs}  )
%          ( M_{mn}^{sc}   M_{mn}^{ss}  )
% We refer to Eq 7.6 --> 7.9 for the definitions of M_{mn}^{cs} in the book: 
% "Mathematical and Statistical Methods for Multistatic Imaging"
%
% D is a C2boundary
% lambda \in \C is the energy. The operator to invert is (lambda - Kstar(D))
% m,n \in \N are the two degrees of polynomials.

%% make A 
nbPoints = D.nbPoints;
Ks = KDstar(D);
As = lambda*eye(nbPoints) - Ks;

%% make nablaPm and Pn
% For nablaPm, we want
% \nabla (x1 + i x2)^m = [1,i]' m (x1 + i x2)^{m-1} 

z = D.points(1,:) + 1i*D.points(2,:);
nablaPm = m*[1,-1i]'*(z.^(m-1));
nablaPmc = real(nablaPm);
nablaPms = imag(nablaPm);

% For Pn, we simply want (x1 + i x2)^n
Pn = z.^n;
Pnc = real(Pn);
Pns = imag(Pn);

% Calculate phic and phis (see Equation 4.1)
phic = As\(D.normal(1,:).*nablaPmc(1,:) + D.normal(2,:).*nablaPmc(2,:))';
phis = As\(D.normal(1,:).*nablaPms(1,:) + D.normal(2,:).*nablaPms(2,:))';

%% Calulate M
M = zeros(2,2);
M(1,1) = (Pnc.*phic')*D.sigma';
M(1,2) = (Pns.*phic')*D.sigma';
M(2,1) = (Pnc.*phis')*D.sigma';
M(2,2) = (Pns.*phis')*D.sigma';

